class Command:
    ADD_COOKIE = 'addCookie'
    ADD_CREDENTIAL = 'addCredential'
    ADD_VIRTUAL_AUTHENTICATOR = 'addVirtualAuthenticator'
    CLEAR_ELEMENT = 'clearElement'
    CLICK_ELEMENT = 'clickElement'
    CLOSE = 'close'
    CONTEXT_HANDLES = 'getContextHandles'
    CURRENT_CONTEXT_HANDLE = 'getCurrentContextHandle'
    DELETE_ALL_COOKIES = 'deleteAllCookies'
    DELETE_COOKIE = 'deleteCookie'
    DELETE_SESSION = 'deleteSession'
    ELEMENT_SCREENSHOT = 'elementScreenshot'
    EXECUTE_ASYNC_SCRIPT = 'executeAsyncScript'
    FIND_CHILD_ELEMENT = 'findChildElement'
    FIND_CHILD_ELEMENTS = 'findChildElements'
    FIND_ELEMENT = 'findElement'
    FIND_ELEMENTS = 'findElements'
    FIND_ELEMENTS_FROM_SHADOW_ROOT = 'findElementsFromShadowRoot'
    FIND_ELEMENT_FROM_SHADOW_ROOT = 'findElementFromShadowRoot'
    FULLSCREEN_WINDOW = 'fullscreenWindow'
    GET = 'get'
    GET_ALL_COOKIES = 'getCookies'
    GET_AVAILABLE_LOG_TYPES = 'getAvailableLogTypes'
    GET_COOKIE = 'getCookie'
    GET_CREDENTIALS = 'getCredentials'
    GET_CURRENT_URL = 'getCurrentUrl'
    GET_ELEMENT_ARIA_LABEL = 'getElementAriaLabel'
    GET_ELEMENT_ARIA_ROLE = 'getElementAriaRole'
    GET_ELEMENT_ATTRIBUTE = 'getElementAttribute'
    GET_ELEMENT_PROPERTY = 'getElementProperty'
    GET_ELEMENT_RECT = 'getElementRect'
    GET_ELEMENT_TAG_NAME = 'getElementTagName'
    GET_ELEMENT_TEXT = 'getElementText'
    GET_ELEMENT_VALUE_OF_CSS_PROPERTY = 'getElementValueOfCssProperty'
    GET_LOG = 'getLog'
    GET_NETWORK_CONNECTION = 'getNetworkConnection'
    GET_PAGE_SOURCE = 'getPageSource'
    GET_SCREEN_ORIENTATION = 'getScreenOrientation'
    GET_SHADOW_ROOT = 'getShadowRoot'
    GET_TIMEOUTS = 'getTimeouts'
    GET_TITLE = 'getTitle'
    GET_WINDOW_RECT = 'getWindowRect'
    GO_BACK = 'goBack'
    GO_FORWARD = 'goForward'
    IS_ELEMENT_ENABLED = 'isElementEnabled'
    IS_ELEMENT_SELECTED = 'isElementSelected'
    MINIMIZE_WINDOW = 'minimizeWindow'
    NEW_SESSION = 'newSession'
    NEW_WINDOW = 'newWindow'
    PRINT_PAGE = 'printPage'
    QUIT = 'quit'
    REFRESH = 'refresh'
    REMOVE_ALL_CREDENTIALS = 'removeAllCredentials'
    REMOVE_CREDENTIAL = 'removeCredential'
    REMOVE_VIRTUAL_AUTHENTICATOR = 'removeVirtualAuthenticator'
    SCREENSHOT = 'screenshot'
    SEND_KEYS_TO_ELEMENT = 'sendKeysToElement'
    SET_NETWORK_CONNECTION = 'setNetworkConnection'
    SET_SCREEN_ORIENTATION = 'setScreenOrientation'
    SET_TIMEOUTS = 'setTimeouts'
    SET_USER_VERIFIED = 'setUserVerified'
    SET_WINDOW_RECT = 'setWindowRect'
    SWITCH_TO_CONTEXT = 'switchToContext'
    SWITCH_TO_FRAME = 'switchToFrame'
    SWITCH_TO_PARENT_FRAME = 'switchToParentFrame'
    SWITCH_TO_WINDOW = 'switchToWindow'
    UPLOAD_FILE = 'uploadFile'
    W3C_ACCEPT_ALERT = 'w3cAcceptAlert'
    W3C_ACTIONS = 'actions'
    W3C_CLEAR_ACTIONS = 'clearActionState'
    W3C_DISMISS_ALERT = 'w3cDismissAlert'
    W3C_EXECUTE_SCRIPT = 'w3cExecuteScript'
    W3C_EXECUTE_SCRIPT_ASYNC = 'w3cExecuteScriptAsync'
    W3C_GET_ACTIVE_ELEMENT = 'w3cGetActiveElement'
    W3C_GET_ALERT_TEXT = 'w3cGetAlertText'
    W3C_GET_CURRENT_WINDOW_HANDLE = 'w3cGetCurrentWindowHandle'
    W3C_GET_WINDOW_HANDLES = 'w3cGetWindowHandles'
    W3C_MAXIMIZE_WINDOW = 'w3cMaximizeWindow'
    W3C_SET_ALERT_VALUE = 'w3cSetAlertValue'